#!/usr/bin/perl

#
# Copyright 2011 Avaya Inc. All Rights Reserved.
#

# Usage:        buildInventoryFile.pl [-s <schema file>] <LAN1> <LAN2> <sys name> <inventory file>
#               buildInventoryFile.pl [-s <schema file>] -l <inventory file>
# Purpose:      Write B5800L XML-formatted high level or low level inventory data to specified file
#
# Description for command line arguments:
# LAN1           - the Ethernet device used as LAN1 (needed only for high level inventory)
# LAN2           - the Ethernet device used as LAN2 (needed only for high level inventory)
# sys name       - the system name (as set through the ABG Manager, needed only for high
#                  level inventory)
# inventory file - the output file name (where the inventory data will be written)
#
# Descriptions for options:
# -s: Receives the path to the upgrade inventory schema file
#     (default upgrade inventory schema file: /usr/lib/ipoffice/CurrentInv.xsd)
# -l: Instructs the script to generate low level inventory

use strict;

use Getopt::Std;
use XML::LibXML;

use vars qw($shortName $usage $outFileName $schemaFileName $lan1Dev $lan2Dev $sysName);
use vars qw($doc $xmlschema $cInventory $inventory $releaseFile $prodVersionName);

umask 0000;

# Creates current inventory document structure with no high level or low level
# inventory data.
#
# Parameters
# None
#
# Returns
# - a handle for the new document in scalar context
# - a list consisting of the new document handle and a reference to the current_inventory
#   node in list context
sub buildEmptyInventory {
  my $invDoc = XML::LibXML::Document->createDocument( "1.0", "UTF-8" );

  # Create response element and set it as the root of the newly created document
  my $root = $invDoc->createElement("response");
  $root->setAttribute( "status", "1" );
  $invDoc->setDocumentElement($root);

  # Append the data, ws_object, upgrade and current_inventory nodes
  my $chunk = <<EOF;
<data>
<ws_object>
<ipo_object>
<upgrade>
<current_inventory>
</current_inventory>
</upgrade>
</ipo_object>
</ws_object>
</data>
EOF

  $root->appendWellBalancedChunk($chunk);

  # If the caller wants a reference to the current_inventory node, provide it
  if (wantarray) {
    my $cInvNode;
    # Find the current_inventory node
    foreach my $node ($invDoc->getElementsByTagName("current_inventory")) {
      $cInvNode = $node;
    }

    return ($invDoc, $cInvNode);
  }

  return $invDoc;
}

# Inserts lan configuration data into given netInfo node.
#
# Parameters
# parent - the existing netInfo node
# index  - 1 or 2 (LAN1 or LAN2)
# lan    - the Linux Ethernet device
#
# Returns
# Nothing
sub insertLanData {
  my ($parent, $index, $lan) = @_;

  # Set interface name
  $parent->appendTextChild("interface", "lan$index");

  my ($mac, $ip, $mask) = ("00-00-00-00-00-00", "0.0.0.0", "0.0.0.0");

  if ($lan ne "") {
    if (open(IFCFG, "/sbin/ifconfig $lan|")) {
      while (<IFCFG>) {
        if (m/HWaddr\s+(\S+)\s*/) {
          $mac = $1;
          next;
        }
        if (m/inet\s+addr:(\S+).+Mask:(\S+)\s*/) {
          $ip = $1;
          $mask = $2;
          last
        }
      }
      close IFCFG;
    }
  }

  $parent->appendTextChild("macAddr", $mac);
  $parent->appendTextChild("ipAddr", $ip);
  $parent->appendTextChild("ipMask", $mask);
}

# Inserts network configuration data into given mainUnit node.
#
# Parameters
# parent - the existing mainUnit node
# lan1   - the Linux Ethernet device used as LAN1
# lan2   - the Linux Ethernet device used as LAN2
#
# Returns
# Nothing
sub insertNetInfo {
  my ($parent, $lan1, $lan2) = @_;

  # Add netInfo node for lan1
  my $element = XML::LibXML::Element->new("netInfo");
  $parent->addChild($element);

  insertLanData($element, 1, $lan1);

  # Add netInfo node for lan2
  my $element = XML::LibXML::Element->new("netInfo");
  $parent->addChild($element);

  insertLanData($element, 2, $lan2);
}

# Inserts lan configuration data into given mainUnit node.
#
# Parameters
# parent - the existing mainUnit node
#
# Returns
# Nothing
sub insertVersionInfo {
  my $parent = shift;

  use vars qw($releaseFile $prodVersionName);

  # Add verInfo node
  my $element = XML::LibXML::Element->new("verInfo");
  $parent->addChild($element);

  # Add version data
  my ($maj, $min, $build) = ("0", "0", "0");
  if (-r $releaseFile) {
    if (open(RELEASE, "<$releaseFile")) {
      while (<RELEASE>) {
        if (m@^$prodVersionName=(\d+)\.(\d+)\.(\d+)\\\((\d+)@) {
          $maj = $1;
          $min = $2;
          $build = "$3.$4";
        }
      }
      close RELEASE;
    }
  }

  $element->appendTextChild("majorVer", $maj);
  $element->appendTextChild("minorVer", $min);
  $element->appendTextChild("buildVer", $build);
}

# Inserts high level inventory data into given current_inventory node.
#
# Parameters
# parent  - the existing current_inventory node
# lan1    - the Linux Ethernet device used as LAN1
# lan2    - the Linux Ethernet device used as LAN2
# sysName - the system name
#
# Returns
#  A reference to the high_level node
sub insertHighLevelInventory {
  my ($parent, $lan1, $lan2, $sysName) = @_;
  my $highLevelNode;

  # Add high_level node
  my $element = XML::LibXML::Element->new("high_level");
  $parent->addChild($element);
  $highLevelNode = $element;

  # Add mainUnit node
  $element = XML::LibXML::Element->new("mainUnit");
  $highLevelNode->addChild($element);

  # Add network interface configuration data
  insertNetInfo($element, $lan1, $lan2);

  # Add platform specific attributes
  $element->appendTextChild("sysName", $sysName);
  $element->appendTextChild("isOffLine", "false");
  $element->appendTextChild("sysType", "IP Office for Linux");
  $element->appendTextChild("sysClass", "CPU");
  $element->appendTextChild("sysMode", "Branch");

  # Add version information
  insertVersionInfo($element);

  return $highLevelNode;
}

# Inserts low inventory data into given entrySet node.
#
# Parameters
# parent  - the existing entrySet node
# pkgName - the RPM package name
# pkgVer  - the RPM package version
# pkgSize - the RPM package size
# pkgTime - the RPM package build time
#
# Returns
# Nothing
sub insertLowLevelEntry {
  my ($parent, $pkgName, $pkgVer, $pkgSize, $pkgTime) =@_;

  # Add per package entry node
  my $element = XML::LibXML::Element->new("entry");
  $parent->addChild($element);

  $element->appendTextChild("fNm", $pkgName);
  $element->appendTextChild("fSz", sprintf("%08x", $pkgSize));
  $element->appendTextChild("fV", $pkgVer);
  $element->appendTextChild("dT", sprintf("%08x", $pkgTime));
}

# Inserts Low level inventory data into given current_inventory node.
#
# Parameters
# parent  - the existing current_inventory node
#
# Returns
#  A reference to the low_level node
sub insertLowLevelInventory {
  my $parent = shift;
  my $lowLevelNode;

  # Add low_level node
  my $element = XML::LibXML::Element->new("low_level");
  $parent->addChild($element);
  $lowLevelNode = $element;

  # Add entrySet node
  $element = XML::LibXML::Element->new("entrySet");
  $lowLevelNode->addChild($element);

  # Add tgtRealPath child to the entrySet
  $element->appendTextChild("tgtRealPath", "/");

  # Get package info and append one entry to the entry set for each package
  my $rpmCmd = 'rpm -qa --queryformat "%{NAME} %{VERSION}%{RELEASE} %{FSSIZES} %{BUILDTIME}\n" 2>/dev/null';
  if (open(PKGDATA, "$rpmCmd|")) {
    while (<PKGDATA>) {
      m/^(\S+)\s+(\S+)\s+(\S+)\s+(\S+)$/;
      insertLowLevelEntry($element, $1, $2, $3, $4);
    }
    close PKGDATA;
  }

  return $lowLevelNode;
}

# Inserts Low level inventory data into given current_inventory node.
#
# Parameters
# statusFileNamePrefix - status file name prefix
# status - status message
#
# Returns
# Nothing
sub publishStatus {
  my ($statusFileNamePrefix, $status) = @_;

  my $statusFileName = "${statusFileNamePrefix}_status";

  unless (open UWSLOCK, ">$statusFileName.lock") {
    print STDERR "$shortName: Unable to open UWS lock file \"$statusFileName.lock\": $1";
    exit(1);
  }

  # Write to the status file while holding an exclusive lock on it
  flock UWSLOCK, 2;

  unless (open UWSSTATUS, ">$statusFileName") {
    print STDERR "$shortName: Unable to open UWS status file \"$statusFileName\": $1";
    flock UWSLOCK, 8;
    close UWSLOCK;
    exit(2);
  }
  print UWSSTATUS "$status\n";
  close UWSSTATUS;

  flock UWSLOCK, 8;
  close UWSLOCK;
}

## Main execution thread starts here

$shortName = eval { my @path = (split "/", $0) ; pop @path; };

$usage = <<EOF;
  Usage: $shortName [-s <schema file>] <LAN1> <LAN2> <sysName> <inventory file>
         - OR -
         $shortName [-s <schema file>] -l <inventory file>
Notes:
 - the default upgrade inventory schema file is: /usr/lib/ipoffice/CurrentInv.xsd
EOF

# Advertise option vars
use vars qw($opt_l $opt_s);

&getopts('ls:');

if (defined($opt_l)) {
  if (@ARGV != 1) {
    print STDERR $usage;
    exit 3;
  }

  $outFileName = $ARGV[0];

} else {
  if (@ARGV != 4) {
    print STDERR "Invalid number of arguments!\n";
    print STDERR $usage;
    exit 3;
  }

  # Get command line parameters
  $lan1Dev = $ARGV[0];
  $lan2Dev = $ARGV[1];
  $sysName = $ARGV[2];
  $outFileName = $ARGV[3];
}

# Use given schema file, if available
if (defined $opt_s) {
  $schemaFileName = $opt_s;
} else {
  $schemaFileName = "/usr/lib/ipoffice/CurrentInv.xsd"
}

# Avaya_release not currently used(may be in the future)
if ( -e "/opt/Avaya/ABG_release" ){
     $releaseFile = "/opt/Avaya/ABG_release";
     $prodVersionName = "ABG_VERSION";
}
elsif ( -e "/opt/Avaya/ABE_release" ){
     $releaseFile = "/opt/Avaya/ABE_release";
     $prodVersionName = "ABE_VERSION";
}
elsif ( -e "/opt/Avaya/APC_release" ){
     $releaseFile = "/opt/Avaya/APC_release";
     $prodVersionName = "APC_VERSION";
}
elsif ( -e "/opt/Avaya/Avaya_release" ){
     $releaseFile = "/opt/Avaya/Avaya_release";
     $prodVersionName = "AVAYA_VERSION";
}

#
# Build inventory data
#
publishStatus($outFileName, "In progress");

# First create the inventory document structure
($doc, $cInventory) = buildEmptyInventory();

if (defined($opt_l)) {
  # Append low level inventory
  $inventory = insertLowLevelInventory($cInventory);
} else {
  # Append high level inventory
  $inventory = insertHighLevelInventory($cInventory, $lan1Dev, $lan2Dev, $sysName);
}

# Validate inventory data, just in case
eval { $xmlschema = XML::LibXML::Schema->new( location => $schemaFileName ); };
if ( $@ ) {
  publishStatus($outFileName, "Failed: could not load schema file:  $schemaFileName");
  exit 4;
}

eval { $xmlschema->validate( $doc ); };
if ( $@ ) {
  chop $@;
  publishStatus($outFileName,
                "Failed: inventory data is not valid according to $schemaFileName schema: $@");
  exit 5;
}

# Write Inventory data to file
unless (open(OUTFILE, ">$outFileName")) {
  publishStatus($outFileName, "Failed: to open output file for writing: $@");
  exit 6;
}

print OUTFILE $inventory->toString(1);
close OUTFILE;

publishStatus($outFileName, "Completed");

