#!/usr/bin/perl

#
# Copyright 2011 Avaya Inc. All Rights Reserved.
#

# Usage:        createUwsArchive.pl <schema file> <release ISO>/<mounted ISO path> <archive name>/<destination repository> <"1"-input is a iso file/"2"-input is mounted iso path> <"abgl"/"abe"/"apc">
# Purpose:      1)Create a ZIP archive suitable for ABGL/ABE software upgrade via SMGR and UWS
#               2)Create file repository on local server based on abg/abe/apc iso dvd
#
# Description for command line arguments:
# schema file  - XML schema definition for the catalog file embedded in the ISO image
# release ISO  - ABGL/ABE software release ISO image (mandatory option "1")
# mounted ISO  - the full path where ISO is mounted  (mandatory option "2")
# archive name - the full path for the generated ZIP archive
# destination repository - the full path to local repository
# build type   - abe:  Avaya Midsize Edition/abgl: Avaya Branch Gateway for Linux/apc: Applications Server on UCM
#

use strict;

use File::Temp qw/ tempfile tempdir /;
use XML::LibXML;

use vars qw($shortName $usage $isoFileName $schemaFileName $archiveFileName $repoDirName @repos);
use vars qw($mountPoint $contentDir $releaseFile $release $buildDate $doc $xmlschema $entrySet);
use vars qw($productType $optionCmd $TYPE_VERSION $isoMountPath $repositoryPath $TargetDIR);

umask 0000;

# Creates UWS catalog document structure.
#
# Parameters
# sessionID - the session ID to be included in the catalog file
#
# Returns
# - a handle for the new document in scalar context
# - a list consisting of the new document handle and a reference to the entrySet
#   node in list context
sub buildEmptyUwsCatalog {
  print STDOUT "buildEmptyUwsCatalog start \n";
  my ($sessionID, $repoDir) = @_;
  my $invDoc = XML::LibXML::Document->createDocument( "1.0", "UTF-8" );
  my $title = "";

  if ( "$productType" eq "abgl" ) {
      $title = "ABG B5800 for Linux - release catalog";
  }
  elsif ( "$productType" eq "abe" ) {
      $title = "Avaya Midsize Edition - release catalog";
  }
  elsif ( "$productType" eq "apc" ) {
      $title = "Applications Server on UCM - release catalog";
  }

  # Create inventory element and set it as the root of the newly created document
  my $root = $invDoc->createElement("inventory");
  $invDoc->setDocumentElement($root);

  # Append the data, ws_object, upgrade and entrySet nodes
  my $chunk = <<EOF;
<sId>$sessionID</sId>
<invTitle>$title</invTitle>
<invType>upgrade</invType>
<entrySet>
<urlPath>$repoDir</urlPath>
</entrySet>
EOF

  $root->appendWellBalancedChunk($chunk);

  # If the caller wants a reference to the entrySet node, provide it
  if (wantarray) {
    my $cInvNode;
    # Find the entrySet node
    foreach my $node ($invDoc->getElementsByTagName("entrySet")) {
      $cInvNode = $node;
    }
    print STDOUT "buildEmptyUwsCatalog end \n";
    return ($invDoc, $cInvNode);
  }
  print STDOUT "buildEmptyUwsCatalog end \n";
  return $invDoc;
}

# Inserts entry data into given entrySet node.
#
# Parameters
# parent  - the existing entrySet node
# name    - entry name (translates to sub-directory under archive directory pointed by
#           entry set URL)
# date    - build date
# id      - entry ID
# comment - entry description
#
# Returns
#  A reference to the entry node
sub insertUwsEntry {
  my ($parent, $date, $name, $id, $comment) = @_;

  # Add high_level node
  my $element = XML::LibXML::Element->new("entry");
  $parent->addChild($element);

  # Add entry specific attributes
  $element->appendTextChild("fName", $name);
  $element->appendTextChild("dT", $date);
  $element->appendTextChild("iSet", $id);
  $element->appendTextChild("cmt", $comment);

  print STDOUT "insertUwsEntry end \n";
  return $element;
}

# Get full filepath for type release.
#
# Global Parameters
# productType - abe/abgl/apc
# mountPoint  - full path where iso is mounted
#
# Output
#  path to release file
#  Avaya_relese could be used in the future. Create premises to work with it
sub getReleaseFile {
    if ( "$productType" eq "abgl" ) {
            $releaseFile = "${mountPoint}/avaya/ABG_release";
         }
    elsif ( "$productType" eq "abe" ) {
            if ( -e "${mountPoint}/avaya/ABE_release" ){
                 $releaseFile = "${mountPoint}/avaya/ABE_release";
            }
            elsif ( -e "${mountPoint}/avaya/Avaya_release" ){
                 $releaseFile = "${mountPoint}/avaya/Avaya_release";
            }
    }
    elsif ( "$productType" eq "apc" ) {
            if ( -e "${mountPoint}/avaya/APC_release" ){
                 $releaseFile = "${mountPoint}/avaya/APC_release";
            }
            elsif ( -e "${mountPoint}/avaya/Avaya_release" ){
                 $releaseFile = "${mountPoint}/avaya/Avaya_release";
            }
    }
    print STDOUT "getReleaseFile: $releaseFile\n";
}

# Create temporary/permanent folders and verify that release file is a proper one.
#
# Global Parameters 
# optionCmd   - 1-input is an iso file /2-input is a path 
# mountPoint  - full path where iso is mounted (temporary for option 1/permanent for option 2)
# contentDir  - destination folder where data is collected
#
# Output
#  initialize parameters
sub createValidateSourceISO {
    print STDOUT "createValidateSourceISO start\n";
    if ( $optionCmd == 1 ) {

         $contentDir = tempdir( "uwsArchiveContentXXXXXX", TMPDIR => 1 );
         unless (-d $contentDir) {
                 print STDERR "Could not create temporary directory to serve as archive content storage\n";
                 exit 6;
         }

         $mountPoint = tempdir( "uwsArchiveMountXXXXXX", TMPDIR => 1 );
         unless (-d $mountPoint) {
                rmdir $contentDir;
                print STDERR "Could not create temporary directory to serve as ISO mount point\n";
                exit 7;
         }

         system("sudo mount -o loop $isoFileName $mountPoint");
         # Check that the "mount" command executed successfully and returned 0
         unless (($? != -1) && (($? & 127) == 0) && (($?>>8) == 0)) {
                rmdir $contentDir;
                rmdir $mountPoint;
                print STDERR "Could not mount $isoFileName\n";
                exit 8;
         }

         getReleaseFile();

         # Validate ISO type
         unless (-r $releaseFile) {
                system("sudo umount $mountPoint");
                rmdir $contentDir;
                rmdir $mountPoint;
                print STDERR "No appropriate release file was found in ISO image: $isoFileName\n";
                exit 9;
         }

    }
    elsif ( $optionCmd == 2 ) {

         $contentDir = $repositoryPath;
         $mountPoint = $isoMountPath;

         getReleaseFile();

         # Validate ISO type
         unless (-r $releaseFile) {
                print STDERR "No appropriate release file was found on mounted ISO image: $isoMountPath\n";
                exit 10;
         }
    }
    print STDOUT "createValidateSourceISO contentDir:$contentDir\n";
    print STDOUT "createValidateSourceISO mountPoint:$mountPoint\n";
    print STDOUT "createValidateSourceISO end\n";
}

# Initialize repository identifiers.
#
# Global Parameters 
# TYPE_VERSION   - ABE/ABG_VERSION located in ABG/ABR_release file 
# repos          - repository identifiers
#
# Output
#  initialize parameters
sub setRepositoryDataType {

   if( "$productType" eq "abe"){
     $TYPE_VERSION="ABE_VERSION";
     @repos = ({'name' => 'system',
               'ID' => 'system',
               'desc' =>"Avaya Midsize Edition - YUM System Updates"},
              {'name' => 'apps',
               'ID' => 'apps',
               'desc' =>"Avaya Midsize Edition - Application Updates"},
              {'name' => 'winapps',
               'ID' => 'winapps',
               'desc' => "Avaya Midsize Edition - Windows Clients Updates"}
              );

   }
   elsif( "$productType" eq "abgl"){
     $TYPE_VERSION="ABG_VERSION";
     @repos = ({'name' => 'system',
               'ID' => 'system',
               'desc' =>"ABG B5800 for Linux - YUM System Updates"},
              {'name' => 'apps',
               'ID' => 'apps',
               'desc' =>"ABG B5800 for Linux - Application Updates"},
              {'name' => 'winapps',
               'ID' => 'winapps',
               'desc' => "ABG B5800 for Linux - Windows Clients Updates"}
              );
   }
   elsif( "$productType" eq "apc"){
     $TYPE_VERSION="APC_VERSION";
     @repos = ({'name' => 'system',
               'ID' => 'system',
               'desc' =>"Applications Server on UCM - YUM System Updates"},
              {'name' => 'apps',
               'ID' => 'apps',
               'desc' =>"Applications Server on UCM - Application Updates"},
              {'name' => 'winapps',
               'ID' => 'winapps',
               'desc' => "Applications Server on UCM - Windows Clients Updates"}
              );
   }
   print STDOUT "setRepositoryDataType productType:$productType type_version:$TYPE_VERSION\n";
}

# Read build version from release file.
#
# Global Parameters 
# releasFile - ABG/ABE/APC_release file
#
# Output
#  release   - release build number
#  buildDate - release build date
sub readTypeVersion {
      print STDOUT "readTypeVersion\n";
      $release="Unknown";
      $buildDate="0000000000000000";
      if (open(RELEASE, "<$releaseFile")) {
        while (<RELEASE>) {
          if (m@^$TYPE_VERSION=(\d+\.\d+\.\d+\\\(\d+)@) {
            $release = $1;
            $release =~ s@\\\(@-@;
          }
          if (m/^BUILD_DATE=(\d+-\d+-\d+_\d+\:\d+)/) {
            $buildDate = $1;
            $buildDate =~ s/[-_:]//g;
          }
        }
        close RELEASE;
      }
     print STDOUT "readTypeVersion $release $TYPE_VERSION\n";
}

# Validate xml schema .
#
# Global Parameters 
# schema file - ARGV[0]
#
# Output
#  success/error
sub validateCatalogData {
    print STDOUT "validateCatalogData\n";
    eval { $xmlschema->validate( $doc ); };

    if ( $@ ) {

      chop $@;
      if ( "$productType" eq "abgl" ) {
         system("sudo umount $mountPoint");
         rmdir $contentDir;
         rmdir $mountPoint;
      }
      elsif ( "$productType" eq "abe" ) {
      }
      elsif ( "$productType" eq "apc" ) {
      }
      print STDERR "Failed: upgrade catalog data is not valid according to $schemaFileName schema: $@";
      exit 11;
    }
}

# Generate xml schema file.
#
# Global Parameters 
# optionCmd  - 1-input is an iso file /2-input is a path
# contentDir - destination folder where data is collected
# release    - build number 
#
# Output
#  UpPackInv.xml file
sub writeCatalogData {
    print STDOUT "writeCatalogData\n";
    if ( $optionCmd == 1 ) {
          eval { $doc->toFile("$contentDir/UpPackInv.xml", 1); };
    }
    elsif ( $optionCmd == 2 ) {
          eval { $doc->toFile("$contentDir/$release/Linux/UpPackInv.xml", 1); };
    }
    if ($@) {

       if ( $optionCmd == 1 ){
           system("sudo umount $mountPoint");
           rmdir $contentDir;
           rmdir $mountPoint;
       }

       print STDERR "Failed to write upgrade catalog data to output file: $@\n";
       exit 13;
    }
}

$shortName = eval { my @path = (split "/", $0) ; pop @path; };

$usage = <<EOF;
  Usage: $shortName <schema file> <release ISO>/<ISO mount path>  <archive name>/<repository path>  <option:"1" -using ISO image file/"2" -using mounted ISO image> <type: "abe"-avaya ip office midsize edition; "abgl"-avaya branch gateway for Linux; "apc"-applications server on UCM>
EOF

if (@ARGV != 5) {
  print STDERR "Invalid number of arguments!\n";
  print STDERR $usage;
  exit 1;
}

$schemaFileName = $ARGV[0];
$optionCmd = $ARGV[3];
$productType = $ARGV[4];

if ( $optionCmd == 1 ) {
    $isoFileName = $ARGV[1];
    $archiveFileName = $ARGV[2];
    $isoMountPath = "";
    $repositoryPath = "";
}
elsif ( $optionCmd == 2 ) {
    $isoFileName = "";
    $archiveFileName = "";
    $isoMountPath = $ARGV[1];
    $repositoryPath = $ARGV[2];
}

unless ( -e $schemaFileName ) {
  print STDERR "Schema file $schemaFileName does not exist\n";
  exit 2;
}

if ( $optionCmd == 1 ) {
   unless (-e $isoFileName ) {
          print STDERR "ISO file isofile=$isoFileName does not exist\n";
          exit 3;
  }
}

if ( $optionCmd == 2 ) {
   unless (-e $isoMountPath ) {
          print STDERR "ISO mount path $isoMountPath cannot be empty\n";
          exit 4;
   }
}

#set repo data
setRepositoryDataType();

# Load schema file, before starting to create directory structure
eval { $xmlschema = XML::LibXML::Schema->new( location => $schemaFileName ); };
if ( $@ ) {
  print STDERR "Failed: could not load schema file:  $schemaFileName";
  exit 5;
}

#set/validate source Catalog 
createValidateSourceISO();

#read version from file
readTypeVersion();


$repoDirName="UWS_repository";
# First create the upgrade catalog document structure
 ($doc, $entrySet) = buildEmptyUwsCatalog($release, $repoDirName);

# Add repository entries to the catalog file
foreach my $repo_href (@repos) {
   insertUwsEntry($entrySet, $buildDate, $repo_href->{'name'},
                  $repo_href->{'ID'}, $repo_href->{'desc'});
}

# Validate the upgrade catalog data, just in case
validateCatalogData();

$TargetDIR = "";
# Now, start creating archive/repository data
if ( $optionCmd == 1 ) {
   $TargetDIR = "$contentDir/$repoDirName";
   mkdir "$TargetDIR";
}
elsif ( $optionCmd == 2 ) {
   my $DirVERSION = "$release/Linux";
   mkdir "$contentDir/$release";
   if ( -e "${mountPoint}/avaya/ABE_release" ){
          system("cp $mountPoint/avaya/ABE_release $contentDir/$release");
   }
   elsif ( -e "${mountPoint}/avaya/APC_release" ){
          system("cp $mountPoint/avaya/APC_release $contentDir/$release");
   }
   elsif ( -e "${mountPoint}/avaya/Avaya_release" ){
          system("cp $mountPoint/avaya/Avaya_release $contentDir/$release");
   }
   mkdir "$contentDir/$release/Linux";
   mkdir "$contentDir/$release/Preupgrade";
   system("cp -R $mountPoint/preupgrade/* $contentDir/$release/Preupgrade/");
   $TargetDIR = "$contentDir/$DirVERSION/$repoDirName";
   mkdir "$TargetDIR";
}

if ( ! -d $TargetDIR ) {
     print STDOUT "Cannot create $TargetDIR Folder\n";
     exit 12;
}

# Write the upgrade catalog data to file
writeCatalogData();

print STDOUT "TargetDir:$TargetDIR\n";

# Create directory structure for our repositories
foreach my $repo_href (@repos) {
   my $repoName = $repo_href->{'name'};
   mkdir "$TargetDIR/$repoName";
   mkdir "$TargetDIR/$repoName/i386";
   if ( ! -d "$TargetDIR/$repoName" || ! -d "$TargetDIR/$repoName/i386" ) {
      print STDOUT "Cannot create folders!!!\n";
      exit 14;
   }
}

print STDOUT "copying System rpms\n";
# Populate "system" repository packages from the CentOS directory within the
# ABGL/ABE/APC release ISO image
if ( -d "$mountPoint/CentOS" ) {
system("cp $mountPoint/CentOS/*.rpm $TargetDIR/system/i386");
}
elsif ( -d "$mountPoint/Packages" ) {
system("cp $mountPoint/Packages/*.rpm $TargetDIR/system/i386");
}

print STDOUT "copying apps rpms\n";
# Populate "apps" repository packages from the subdirectories of the avaya directory
# within the ABGL/ABE/APC release ISO image
system("cp $mountPoint/avaya/*/*.rpm $TargetDIR/apps/i386");

# There are no Windows applications bundled with the ABGL releases now, leaving
# the winapps repository empty
#if ( "$productType" eq "abe" ) {
#system("mv $TargetDIR/apps/i386/*.exe $TargetDIR/winapps/i386");
#}

# Generate repository metadata for our repositories
foreach my $repo_href (@repos) {
   my $repoName = $repo_href->{'name'};
   system("cd $TargetDIR/$repoName; createrepo .");
}

if ( $optionCmd == 1) {
   print STDOUT "Aip-umount-removeTmpDirs\n";
   # Archive the content into specified archive
   system("cd $contentDir; zip -r - . > $archiveFileName");

   # Unmount ISO image, cleanup temporaty data
   system("sudo umount $mountPoint");
   rmdir $mountPoint;
   system("rm -rf $contentDir");
}
elsif ( $optionCmd == 2 ) {
   system("chmod -R a+rX $contentDir/$release");
}

print STDOUT "END createUwsArchive\n";
