#!/usr/bin/perl

#
# Copyright 2011 Avaya Inc. All Rights Reserved.
#

# Usage:        parseUpgradeCatalog.pl [-o <output file>] [-s <schema file>] <base URL> <inventory file> [<session ID>]
# Purpose:      Convert ABG B5800 upgrade catalog data to YUM repository file
#
# Description for command line arguments:
# <base URL>       - the base URL containing the inventory file and upgrade packages
# <inventory file> - the path to the downloaded inventory file
# <session ID>     - optional; if present overrides the session ID present in the catalog file
#
# Descriptions for options:
# -o: Receives the path to the output file as an argument
#     (default output file: /etc/yum.repos.d/AVAYA_UWS_<session ID>.repo)
# -s: Receives the path to the upgrade inventory schema file
#     (default upgrade inventory schema file: /usr/lib/ipoffice/UpPackInv.xsd)

use strict;

use Getopt::Std;
use XML::LibXML;

use vars qw($shortName $usage $outFileName $schemaFileName $baseUrl $invFileName $sessionID);
use vars qw($doc $xmlschema $attr);

umask 0000;

## Main execution thread starts here

$shortName = eval { my @path = (split "/", $0) ; pop @path; };

$usage = <<EOF;
  Usage: $shortName [-o <output file>] [-s <schema file>] <base URL> <inventory file> [<session ID>]
Notes:
 - the default YUM repo file name is: /etc/yum.repos.d/AVAYA_UWS_<session ID>.repo
   (the session ID is extracted from the <sId> tag in the inventory file
 - the default upgrade inventory schema file is: /usr/lib/ipoffice/UpPackInv.xsd
EOF

# Advertise option vars
use vars qw($opt_o $opt_s);

&getopts('o:s:');

if ((@ARGV != 2) && (@ARGV != 3)) {
  print STDERR "Invalid number of arguments!\n";
  print STDERR $usage;
  exit 1;
}

# Get command line parameters
$baseUrl = $ARGV[0];
$invFileName = $ARGV[1];

if (@ARGV == 3) {
  $sessionID = $ARGV[2];
}

# Use given schema file, if available
if (defined $opt_s) {
  $schemaFileName = $opt_s;
} else {
  $schemaFileName = "/usr/lib/ipoffice/UpPackInv.xsd"
}

# Parse and validate received inventory file
eval { $doc = XML::LibXML->new->parse_file($invFileName); };
if ( $@ ) {
  print STDERR "Could not parse input file:  $invFileName\n";
  exit 2;
}

eval { $xmlschema = XML::LibXML::Schema->new( location => $schemaFileName ); };
if ( $@ ) {
  print STDERR "Could not load schema file:  $schemaFileName\n";
  exit 3;
}

eval { $xmlschema->validate( $doc ); };
if ( $@ ) {
  print STDERR "$invFileName is not valid according to $schemaFileName schema\n";
  exit 4;
}

###
### The inventory file was successfully validated against the schema file
###

# Extract the session ID from the inventory file, if not provided as a command line argument
if (! defined($sessionID)) {
  $sessionID = "";
  foreach ($doc->getElementsByTagName("inventory")) {
    foreach $attr ($_->getChildrenByTagName("sId")) {
      $sessionID = $attr->textContent;
    }
  }
}

# Use given output file name if available
if (defined $opt_o) {
  $outFileName = $opt_o;
} else {
  $outFileName = "/etc/yum.repos.d/AVAYA_UWS_${sessionID}.repo";
}

unless (open (OUTFILE, ">$outFileName")) {
  print STDERR "Could not open \"$outFileName\" for writing: $!\n";
  exit 5;
}

# Generate the repository file header
print OUTFILE <<EOF;
# $outFileName
#
# Configuration for IP Office for Linux UWS update repositories
#
EOF

# Generate repository file; the GPG checking will be disabled until proper RPM
# file signing is implemented. All repositories will be disabled, in order to
# avoid interactions with other package upgrade tools
foreach my $entrySet ($doc->getElementsByTagName("entrySet")) {
  my $repoUrl;
  foreach $attr ($entrySet->getChildrenByTagName("urlPath")) {
    $repoUrl = "$baseUrl/" . $attr->textContent;
  }

  foreach my $entry ($entrySet->getChildrenByTagName("entry")) {
    my ($fName, $iSet, $cmt) = ("", "", "", "");

    foreach $attr ($entry->getChildrenByTagName("fName")) {
      $fName = $attr->textContent;
    }

    foreach $attr ($entry->getChildrenByTagName("iSet")) {
      $iSet = $attr->textContent;
    }

    foreach $attr ($entry->getChildrenByTagName("cmt")) {
      $cmt = $attr->textContent;
    }

    print OUTFILE <<EOF;

[${iSet}-${sessionID}]
name=$cmt
baseurl=$repoUrl/$fName
gpgcheck=0
enabled=0
EOF
  }
}

close(OUTFILE);
