#!/bin/bash

# Used by ipoffice service on Linux. Please keep in sync if you need to change it
# -q verified if a package is installed
# -v get version of a package
# -i install a package(!!!no dependecies error checked)
# -r local path
# -f file where output command is redirected
# -c check after install if a package was successful installed

# Exit codes
# successful executed script
RETCODE_SUCCESS=0
# default package found
RETCODE_DEFAULTFOUND=1
# successful installed and checked
RETCODE_DEFAULTINSTALLED=2
# FAILURE RETCODE > 100
# failed to execute script
RETCODE_FAILED=101
# no supported argument passed
RETCODE_NOTSUPPORTEDARGUMENT=102
# no valid argument passed
RETCODE_INVALIDARGUMENT=103
# mutiple versions found for a package
RETCODE_MULTIPLEPACKAGESFOUND=104


if [ -f /opt/Avaya/scripts/config_files/apps_logging_config ]; then
    . /opt/Avaya/scripts/config_files/apps_logging_config
fi

if [ -n ${ISA_LOG_PATH_NAME} -a -f ${ISA_LOG_PATH_NAME} ]; then
   logfile=${ISA_LOG_PATH_NAME}
else
   logfile=/tmp/.rpmpkginfo
fi

function logevent () {
   local arg_l1="`date`"  #date
   local arg_l2=$1        #line n
   local arg_l3=$2        #event

   if [ -f "$logfile" ]; then
       echo "$$ || ${arg_l1} || $0 Line:${arg_l2} || ${arg_l3}"  >> $logfile
   fi
}


while getopts :q:v:i:r:f:c: option
do
   case "${option}" in
        q) qpackagename=${OPTARG};;
        c) cpackagename=${OPTARG};;
        v) vpackagename=${OPTARG};;
        i) ipackagename=${OPTARG};;
        r) localrepo=${OPTARG};;
        f) file=${OPTARG};;
        \?) echo "Invalid option: -$OPTARG" >&2
            logevent "$LINENO" "invalid option: -$OPTARG"
            exit $RETCODE_NOTSUPPORTEDARGUMENT;;
   esac
done

exp='^[-~!@#$%^&*+]+[a-zA-Z]*$'

if [ -z "$qpackagename" -a -z "$ipackagename" -a -z "$vpackagename" -a -z "$localrepo" ]; then
   echo "Error: Empty package name is not allowed!"
   logevent "$LINENO" "error: empty package name is not allowed!"
   exit $RETCODE_NOTSUPPORTEDARGUMENT
fi

if [[ $ipackagename =~ $exp || $qpackagename =~ $exp || $vpackagename =~ $exp || $localrepo =~ $exp || $cpackagename =~ $exp || $file =~ $exp ]]; then
   echo "Error: No valid package name format!"
   logevent "$LINENO" "error: no valid package name format!"
   exit $RETCODE_INVALIDARGUMENT
fi

if [ -n "$qpackagename" ]; then
   cmd=$(rpm -q "$qpackagename")
   if [ -n "$file" ]; then
      echo "$cmd" > $file
      if [[ "$cmd" =~ "1.0.0-1" ]]; then
         for i in $cmd
         do
             if [[ "$i" != *"1.0.0-1"* ]]; then
                logevent "$LINENO" "errorcode $RETCODE_MULTIPLEPACKAGESFOUND"
                exit $RETCODE_MULTIPLEPACKAGESFOUND
             fi
         done
         logevent "$LINENO" "default server type was found. retcode $RETCODE_DEFAULTFOUND"
         exit $RETCODE_DEFAULTFOUND
      fi
   fi
elif [ -n "$vpackagename" ]; then
   cmd=$(rpm -q --queryformat '%{VERSION}-%{RELEASE}\n' "$vpackagename")
   if [ -n "$file" ]; then
      echo "$cmd" > $file
      logevent "$LINENO" "package version $cmd"
   fi
elif [ -n "$ipackagename" ]; then
   cmd=$(rpm -q $cpackagename)
   if [[ "$cmd" =~ "is not installed" ]]; then
      if [ -n "$file" ]; then
         cmd=$(rpm -i $localrepo$ipackagename)
         if [ -n "$cpackagename" ]; then
            cmd=$(rpm -q $cpackagename)
            echo "$cmd" > $file
            if [[ "$cmd" =~ "1.0.0-1" ]]; then
               logevent "$LINENO" "default server type was installed. retcode $RETCODE_DEFAULTINSTALLED"
               exit $RETCODE_DEFAULTINSTALLED
            fi
         fi
      else
         cmd=$(rpm -i $localrepo$ipackagename)
         logevent "$LINENO" "installing... $cmd"
      fi
   else
      if [[ "$cmd" =~ "1.0.0-1" ]]; then
         for i in $cmd
         do
            if [[ "$i" != *"1.0.0-1"* ]]; then
                logevent "$LINENO" "errorcode $RETCODE_MULTIPLEPACKAGESFOUND"
                exit $RETCODE_MULTIPLEPACKAGESFOUND
            fi
         done
         if [ -n "$file" ]; then
            echo "$cmd" > $file
         fi
         logevent "$LINENO" "default server type was found. retcode $RETCODE_DEFAULTFOUND"
         exit $RETCODE_DEFAULTFOUND
      fi
   fi
fi

if [ -n "$cmd" -a -z "$file" ]; then
   echo "$cmd"
fi

logevent "$LINENO" "retcode $RETCODE_SUCCESS"
exit $RETCODE_SUCCESS
