#!/usr/bin/perl

#
# Copyright 2011 Avaya Inc. All Rights Reserved.
#

#
# Run yum in background to download and/or upgrade packages, while getting progress on the action
# yumUpdateProgress.pl [-c ] [-d ] <session ID>
# Notes:
#  - the default behavior is to upgrade (previously downloaded) package files from the
#    repository corresponding to given session ID
#  - providing the "-c" option will cause a YUM package cache cleanup before any other actions
#  - providing the "-d" option will trigger the "download only" mode and will honor
#    interrupt requests
#
# Dependencies:
# The screen and yum-downloadonly packages have to be installed

use strict;
use File::Temp qw/ :mktemp  /;
use Getopt::Std;
use POSIX qw(strftime);
use Tie::File;
use Fcntl 'O_RDONLY';

use vars qw($shortName $usage $sessionID $yumOutFile $status $yumCmd $yumOpts);
use vars qw($downloadOnlyMode $statusFilePrefix $requestStatusFilePrefix $abortRequested $logFile);
use vars qw($pidFile $yumPidFile $yumSavedLogFile $yumLogPath $HistoryFolder $yumGlobalRepoFilePath);
use vars qw($infosizefile $infosize $sfound $currentTime $started $finished @array);
use vars qw($uwsReportFilePath $uwsReportFile $uwsReportTmpFile $addDataToReportScript);

umask 0000;

sub logFunc {
  my ($status) = @_;
  
  unless (open STATUSLOG, ">>$logFile") {
    print STDERR "$shortName: Unable to open UWS status log file \"status_log\": $1";
    return 0;
  }

  my $dateString=localtime();
  print STATUSLOG "$dateString $status\n";
  close STATUSLOG;
}

sub addToHistory {
  my ($backedFilePath,$backedFileName) = @_;

  if (! -d $backedFilePath) {
     system("sudo mkdir $backedFilePath");
  }

  if (! -d "${backedFilePath}${HistoryFolder}") {
     system("sudo mkdir ${backedFilePath}${HistoryFolder}");
  }

  if (-e "${backedFilePath}${backedFileName}") {
     system("sudo mv ${backedFilePath}${backedFileName}  ${backedFilePath}${HistoryFolder}${backedFileName}_${currentTime}");
  }
}

# Fill the report file. In download mode we save only the errors
sub getProcessedYUMEvents {
  use vars qw($uwsReportFilePath $uwsReportFile $started $finished $downloadOnlyMode);
  my ($outputFileName,$inputFileName) = @_;
  my $state = 0;

  if (-e $inputFileName) {
     tie @array, 'Tie::File', $inputFileName , mode => O_RDONLY;
     ($started,$finished) = @array[0,$#array];
     untie @array;

     if (index($started,"Script started on") == -1){
        $started="";
     }
     if (index($finished,"Script done on") == -1){
        $finished="";
     }
  }

  if ($downloadOnlyMode == 0) {
     system("${addDataToReportScript} -t \"${finished}\" -f ${uwsReportFilePath}${uwsReportFile}");
  }

  unless (open TMPINPUT, "<$inputFileName") {
     print STDERR "Unable to open \"$inputFileName\" for read: $1";
     return;
  }

  unless (open TMPOUTPUT, ">$outputFileName") {
     print STDERR "Unable to open \"$outputFileName\" for write: $1";
     return;
  }
  
  while (<TMPINPUT>) {
     chomp $_;

    if ((m/^Error:/) || (m/^Error /) || (m/^ERROR with rpm_check_debug vs depsolve:/)) {
       print TMPOUTPUT "\nException occurred:\n";
       print TMPOUTPUT "\t$_\n";
       next;
    }
    if (m/^No Packages marked for Update/) {
       print TMPOUTPUT "No Packages marked for Update\n";
       last;
    }
    if (m/^exiting because --downloadonly specified:/) {
       last;
    }
    if (m/^Updated:/) {
       $state = 1;
       print TMPOUTPUT "\nUpdated:\n";
       next;
    }
    if (m/^Dependency Installed:/) {
       $state = 1;
       print TMPOUTPUT "\nDependency Installed:\n";
       next;
    }
    if (m/^Replaced:/) {
       $state = 1;
       print TMPOUTPUT "\nReplaced:\n";
       next;
    }
    if (m/^Failed:/) {
       $state = 1;
       print TMPOUTPUT "\nFailed:\n";
       next;
    }
    if (m/^Complete!/) {
       last;
    }

    if ($state ==1) {
       if (m/^\s*($|#)/){
          next;
       }
       else
       {
          print TMPOUTPUT "\t$_\n";
       }
    }
  }
  close TMPINPUT;
  close TMPOUTPUT;
}

sub publishStatus {
  my ($statusFileNamePrefix, $status) = @_;

  my $statusFileName = "${statusFileNamePrefix}_status";

  unless (open UWSLOCK, ">$statusFileName.lock") {
    print STDERR "$shortName: Unable to open UWS lock file \"$statusFileName.lock\": $1";
    exit(1);
  }

  # Write to the status file while holding an exclusive lock on it
  flock UWSLOCK, 2;

  unless (open UWSSTATUS, ">$statusFileName") {
    print STDERR "$shortName: Unable to open UWS status file \"$statusFileName\": $1";
    flock UWSLOCK, 8;
    close UWSLOCK;
    exit(2);
  }

  print UWSSTATUS " $status\n";
  close UWSSTATUS;

  flock UWSLOCK, 8;
  close UWSLOCK;

  if (index($status, "Downloading packages:") == -1){
      logFunc($status);
  }
}


sub getStatus {
  my ($fileName)=@_;

  use vars qw($downloadOnlyMode $sessionID $abortRequested);

  my $status="Resolving package dependencies";
  #
  # Here is the coding for the states:
  # 0 - Initial state
  # 1 - Downloading packages
  # 2 - Upgrading packages
  # 3 - Done
  # 4 - Process during startup
  my $state=0;
  my $percent;

  my $yumPid = getYumPidForSession($sessionID);
  if ($abortRequested != 0) {
    if ($yumPid != 0) {
      $status="Aborting";
    } else {
      # Update request status file upon detecting that YUM sub-process is gone
      publishStatus($requestStatusFilePrefix, "Aborted");
      $status="Aborted";
    }
    if (-e "${uwsReportFilePath}${uwsReportFile}" && -e $addDataToReportScript) {
       system("${addDataToReportScript} -e \"Abort Requested=${status}\" -f ${uwsReportFilePath}${uwsReportFile}");
    }

    return $status;
  }


  if (-e "$fileName") {
    unless (open YUMOUT, "<$fileName") {
      publishStatus($statusFilePrefix, "Error: Internal error (screen log \"$fileName\" exists but could not be opened): $!\n");
      exit(3);
    }
  }
  else {
    $state=4;
    $percent=0;
    $status="Yum process startup";
    logFunc($status);
    return $status;
  }

  while (<YUMOUT>) {
    if ((m/^Total download size:/) || (m/^Total size:/)) {
      $infosize = $_;
      chomp($infosize);
      if( $infosize =~ m/\x0D$/){
         chop($infosize);
      }
      next;
    }
    if (m/^Script started on/) {
      $started = $_;
      chomp($started);
      next;
    }
    if (m/^Downloading Packages:/) {
      # We are downloading packages at this time
      $state=1;
      $percent=0;
      next;
    }
    if (m/^Transaction Test Succeeded/) {
      if ($downloadOnlyMode == 0) {
        # We are upgrading packages at this time
        $percent=0;
        $state=2;
      }
      next;
    }
    if (m/(Nothing to do)|(No Packages marked for Update)/) {
      $status="No action required";
      $state=3;
      last;
    }
    if (m/Error Downloading Packages\:/) {
      $status="Error: package download";
      $state=3;
      last;
    }
    if (m/(Error: Cannot retrieve repository metadata)|(Error: failure: repodata)/) {
      $status="Error: missing/bad repository metadata";
      $state=3;
      last;
    }
    if (m/(Error: Missing Dependency:)|(ERROR with rpm_check_debug vs depsolve:)/) {
      $status="Error: failed package dependency resolution";
      $state=3;
      last;
    }

    if ($state == 1) {
      if (m/^exiting because --downloadonly specified/) {
	$status="Package download completed";
        $state=3;
	last;
      }
      if (m/^\s*\((\d+)\/(\d+)\)\:/) {
        if ($1 > 1) {
          $percent=sprintf("%.2f", 100 * ($1-1) / $2);
        }
      }
    }
    if ($state == 2) {
      if (m/^Complete!/) {
	$status="Package upgrade completed";
        $state=3;
	last;
      }
      if (m/^Updated:/) {
        $percent=100;
      }
      if (m/\s+(\d+)\/(\d+)\s*$/) {
        if ($1 > 1) {
          $percent=sprintf("%.2f", 100 * ($1-1) / $2);
        }
      }
    }
  }
  close YUMOUT;

  if ($state==1) {
    $status="Downloading packages: $percent% completed";
  }

  if ($state==2) {
    $status="Upgrading packages: $percent% completed";
  }

  if (($yumPid == 0) && ($state != 3)) {
    # According to the output file, YUM didn't finish its job.
    # However, the yum process is gone...
    $status = "Error: yum process died before completing its job";
  
    if (-e "${uwsReportFilePath}${uwsReportFile}" && -e $addDataToReportScript) {
      system("${addDataToReportScript} -e \"${status}\" -f ${uwsReportFilePath}${uwsReportFile}");
    }
  }
  return $status;
}

sub getYumPidForSession {
  my ($sid) = @_;
  my $pid=0;

  open CMDOUT, "ps -ef|";
  while (<CMDOUT>) {
    if (m@/usr/bin/yum\s+.*$sid@) {
      m/^\S+\s+(\d+)/;
      $pid=$1;
      last;
    }
  }
  close CMDOUT;

  return $pid;
}

sub sigIntHandler {
  use vars qw($downloadOnlyMode $requestStatusFilePrefix $sessionID $abortRequested $pidFile);

  unless (defined($downloadOnlyMode)) {
    # The package download didn't even start, update status, do cleanup and exit
    publishStatus($requestStatusFilePrefix, "Aborted");
    unlink($pidFile);
    exit(0);
  }

  if ($downloadOnlyMode == 1) {
    # Update internal state info to account for requesting YUM to stop
    $abortRequested = 1;

    # Initiate yum download abort
    my $yumPid = getYumPidForSession($sessionID);
    if ($yumPid != 0) {
      publishStatus($requestStatusFilePrefix, "Aborting");
      kill 'INT', $yumPid;
    } else {
      publishStatus($requestStatusFilePrefix, "Abort failed");
    }
  } else {
    # Abort only supported in download mode
    publishStatus($requestStatusFilePrefix, "Not interruptible");
  }
}

sub isProcessRunning {
  my ($pid) = @_;
  my $retVal = 0;

  system("ps -p $pid > /dev/null");
  # Check that the "ps" command executed successfully and returned 0
  if (($? != -1) && (($? & 127) == 0) && (($?>>8) == 0)) {
    $retVal = 1;
  }
  return $retVal;
}

sub getPidFromFile {
  my ($fileName) = @_;

  if (!open(FH, "<$fileName")) {
    return 0;
  }
  my $pid=<FH>;
  close FH;

  chomp $pid;

  return ($pid =~ m/^\d+$/) ? $pid : 0;
}

sub updateWcpRepositories {
  my ($sessionID) = @_;

  my ($uwsRepoDir, $wcpRepoRoot) = ("/var/cache/yum/x86_64/6", "/var/cache/avaya");
  my @repos = ("apps", "winapps");

  foreach my $repo (@repos) {
    my $packagesPattern = "$uwsRepoDir/${repo}-${sessionID}/packages/*.rpm";
    my $wcpRepoDir = "$wcpRepoRoot/$repo";

    ### Only do something if package files were actually downloaded
    my @fileList = glob $packagesPattern;
    if (@fileList) {
      system("sudo cp $packagesPattern $wcpRepoDir");
      chdir("$wcpRepoDir");
      system("sudo createrepo . > /opt/Avaya/uws_logs/createrepo_${sessionID}_${repo}.log 2>&1")
    }
  }
}

## Main execution thread starts here

$shortName = eval { my @path = (split "/", $0) ; pop @path; };

$usage = <<EOF;
  Usage: $shortName [-c] [-d ] <session ID>
Notes:
 - the default behavior is to upgrade (previously downloaded) package files from the
   repository corresponding to given session ID
 - providing the "-c" option will cause a YUM package cache cleanup before any other actions
 - providing the "-d" option will trigger the "download only" mode and will honor
   interrupt requests
EOF

# Advertise option vars
use vars qw($opt_c $opt_d);

&getopts('cd');

if (@ARGV != 1) {
  print STDERR "$shortName: Invalid number of arguments!\n";
  print STDERR $usage;
  exit(4);
}

$sessionID = $ARGV[0];
$statusFilePrefix = "/tmp/B5800_UWS";
$requestStatusFilePrefix = "${statusFilePrefix}_request";
$pidFile = "${statusFilePrefix}.pid";
$yumPidFile = "/var/run/yum.pid";
$infosizefile = "${statusFilePrefix}.size";
$infosize = "";
$sfound = 0;
$yumLogPath = "/opt/Avaya/uws_logs/";
$logFile = "${yumLogPath}status_log";
$HistoryFolder = "history/";
$yumGlobalRepoFilePath = "/etc/yum.repos.d/AVAYA_UWS_${sessionID}.repo";
$currentTime = strftime("%Y-%m-%d_%H-%M-%S", localtime());
$uwsReportFile = "upgrade_report.txt";
$uwsReportFilePath = "/opt/ipoffice/system/logs/";
$addDataToReportScript = "/opt/Avaya/scripts/update_uws_report.sh";

if(! -d $uwsReportFilePath) {
  system("sudo mkdir $uwsReportFilePath");
}
system("sudo chmod 775 {/opt/,/opt/ipoffice/,/opt/ipoffice/system/}");
system("sudo chmod 777 ${uwsReportFilePath}");
system("sudo chmod 777 /tmp");
system("sudo chmod 666 ${statusFilePrefix}*");

if (-e $pidFile) {
  my $runningPid=getPidFromFile($pidFile);
  if (($runningPid != 0) && isProcessRunning($runningPid)) {
    print STDERR "$shortName: Detected another running instance (PID=$runningPid); exiting...\n";
    exit(5);
  }
}

# Mark the current process as running instance
if (!open(PIDFILE, ">$pidFile")) {
  print STDERR "$shortName: unable to open \"$pidFile\" PID file for writing: $!\n";
  exit(6);
}
print PIDFILE "$$\n";
close PIDFILE;

#
# We own the status files now, using them for error reporting whenever possible
#

if (-e $logFile) {
  unlink $logFile
}

logFunc("Starting the upgrade process");
publishStatus($requestStatusFilePrefix, "Idle");

if (-e $yumPidFile) {
  my $runningPid=getPidFromFile($yumPidFile);
  if (($runningPid != 0) && isProcessRunning($runningPid)) {
    publishStatus($statusFilePrefix, "Error: YUM running as PID $runningPid");
    unlink($pidFile);
    exit(7);
  }
}

# Install the interrupt handler as early as possible
$SIG{INT}=\&sigIntHandler;

#
# Cleanup potential leftover YUM cached metadata, if requested
#
$yumOpts="-y --disablerepo=\* --enablerepo=system-${sessionID} --enablerepo=apps-${sessionID} --enablerepo=winapps-${sessionID}";
if (defined($opt_c)) {
  system("sudo yum $yumOpts clean all > /dev/null");
}

publishStatus($statusFilePrefix, "Starting package download");

if (defined($opt_d)) {
  $downloadOnlyMode = 1;
} else {
  $downloadOnlyMode = 0;
}

# Before to start downloading packages save preview report uws file
# On upgrade, just update it
if (-e "${uwsReportFilePath}${uwsReportFile}" && $downloadOnlyMode == 1) {
  addToHistory($uwsReportFilePath,$uwsReportFile);
}

# Create new uws report file if no one exist/otherwise just update it
if (! -e "${uwsReportFilePath}${uwsReportFile}") {
  if (!open(REPORTFILE, ">${uwsReportFilePath}${uwsReportFile}")) {
     print STDERR "$shortName: unable to open \"${uwsReportFilePath}${uwsReportFile}\" report file for writing: $!\n";
  }
  else {
     close REPORTFILE;
  }
}

$yumOutFile=mktemp("/tmp/yumOut_XXXXXX");

# Add System information to uws report file
# 1 server type
# 2 ipaddresses configured on server
# 3 pid of operation in progress
if (-e "${uwsReportFilePath}${uwsReportFile}" && -e $addDataToReportScript) {
  if ($downloadOnlyMode == 1) {
     system("${addDataToReportScript} -i infoserver -f ${uwsReportFilePath}${uwsReportFile}");
     system("${addDataToReportScript} -r ${yumGlobalRepoFilePath} -f ${uwsReportFilePath}${uwsReportFile}");
     system("${addDataToReportScript} -v prior_upgrade -f ${uwsReportFilePath}${uwsReportFile}");
  }
}


if ($downloadOnlyMode == 1) {
  $yumOpts .= " --downloadonly";
}
$yumCmd="screen -d -m script -f -c 'trap \"\" HUP; yum $yumOpts update' $yumOutFile&";
system($yumCmd);

my $yum_startup=0;

while (($status ne "Package download completed")
       && ($status ne "Package upgrade completed")
       && ($status ne "No action required")
       && ($status ne "Aborted")
       && ( $status !~ /^Error/)
       && ($yum_startup < 5)
      )
{
  sleep 5;
  $status=getStatus($yumOutFile);
  if (($downloadOnlyMode == 1) && ($status eq "Package download completed")) {
    updateWcpRepositories($sessionID);
  }
  if ($status eq "Yum process startup") {
    ++$yum_startup;
  }
  publishStatus($statusFilePrefix, $status);
  if (($sfound == 0) && $infosize ) {
       if (!open(INFOSIZEFILE, ">$infosizefile")) {
          print STDERR "$shortName: unable to open \"$infosizefile\" file for writing: $!\n";
       }
       else
       {
          $sfound = 1;
          print INFOSIZEFILE "$infosize\n";
          close INFOSIZEFILE;
          if ($downloadOnlyMode == 1) {
             system("${addDataToReportScript} -t \"${started}\" -f ${uwsReportFilePath}${uwsReportFile}");
             system("${addDataToReportScript} -p $$ -o ${downloadOnlyMode} -f ${uwsReportFilePath}${uwsReportFile}");
             system("${addDataToReportScript} -s \"${infosize}\" -f ${uwsReportFilePath}${uwsReportFile}");
          }
          elsif ($downloadOnlyMode == 0) {
             #save for moment only upgrade session id
             system("${addDataToReportScript} -p $$ -o ${downloadOnlyMode} -f ${uwsReportFilePath}${uwsReportFile}");
          }
       }
  }
}

if ($yum_startup == 6) {
  publishStatus($statusFilePrefix, "Error: Yum process is taking too long to start");
  sigIntHandler();
  exit(4);
}

if ($downloadOnlyMode == 1) {
  $yumSavedLogFile = "yum_download_${sessionID}.log"
} else {
  $yumSavedLogFile = "yum_upgrade_${sessionID}.log"
}

# save the old logs to history folder
addToHistory($yumLogPath,$yumSavedLogFile);

if (-e "${uwsReportFilePath}${uwsReportFile}" && -e $addDataToReportScript && $downloadOnlyMode == 0) {
   system("${addDataToReportScript} -v upgrade -f ${uwsReportFilePath}${uwsReportFile}");
}

# temporary file used to keep updated packages
$uwsReportTmpFile=mktemp("/tmp/uwsReport_XXXXXX");
if (!open(RTMPFILE, ">$uwsReportTmpFile")) {
  print STDERR "$shortName: unable to open \"$uwsReportTmpFile\" temp file for writing: $!\n";
}
else {
  close RTMPFILE;
}

if (-e $uwsReportTmpFile && -e $yumOutFile) {
  # save to uws report file
  getProcessedYUMEvents($uwsReportTmpFile,$yumOutFile);
   if (-e "${uwsReportFilePath}${uwsReportFile}") {
      system("sudo dos2unix ${uwsReportTmpFile} &>/dev/null");
      system("sudo /bin/cat ${uwsReportTmpFile} >> ${uwsReportFilePath}${uwsReportFile}");
   }
}

if (-e $uwsReportTmpFile) {
   system("sudo /bin/rm -f $uwsReportTmpFile");
}

system("sudo dos2unix ${yumOutFile} &>/dev/null");
system("sudo mv $yumOutFile ${yumLogPath}${yumSavedLogFile}");
unlink($pidFile);
